<?php
/**
 * @file
 * Defines module classes
 *
 * @ingroup xml_field
 * @{
 */

/*
 * Class xmlFieldXMLElement
 *
 * Allows for proper formatting of CDATA
 *
 * The approach taken is this: any node value that is wrapped in CDATA is
 * stashed in the class static variable, keyed by the instance id, and then
 * later retrieved during the asXml() method.
 */
class xmlFieldXMLElement extends simpleXMLElement {

  /**
   * Do not access this variable; use the set/getConfig methods instead.
   *
   * @var array
   */
  public static $config = array();

  /**
   * Returns a string (or node value) wrapped in CDATA.
   *
   * @code
   *   $xml->addChild('title', '<![CDATA[<h1>My Book</h1>]]>');
   *   $xml->cdata($xml->title);
   * @endcode
   *
   * @param string|XmlFieldXmlElement $key
   */
  public function cdata($key = NULL, $force = FALSE) {
    $value = (string) $key;
    self::wrapCDataIfNeeded($value, $force);

    return $value;
  }

  /**
   * Public setting for the AutoCData setting
   *
   * @param bool $state
   */
  public static function setAutoCData($state) {
    self::setConfig('autoCData', $state);
  }

  /**
   * Add a child element; this corrects CDATA issues
   *
   * @throws RuntimeException
   */
  public function addChild($name, $value = NULL, $namespace = NULL) {

    // Automatic CDATA handling if enabled.
    if ($this->getConfig('autoCData')) {
      $this->wrapCDataIfNeeded($value);
    }

    // Stash a CDATA value for later...
    $valueAsCData = FALSE;
    if (preg_match("/^\<\!\[CDATA\[(.+)?\]\]\>$/", $value, $output_array)) {
      $valueAsCData = $output_array[1];
      $value = NULL;
    }

    $child = parent::addChild($name, $value, $namespace);
    // switch (func_num_args()) {
    //   case 1:
    //     $child = parent::addChild($name);
    //     break;

    //   case 2:
    //     $child = parent::addChild($name, $value);
    //     break;

    //   case 3:
    //     $child = parent::addChild($name, $value, $namespace);
    //     break;
    // }      

    if ($valueAsCData) {
      $node = dom_import_simplexml($child);
      $no   = $node->ownerDocument;
      $node->appendChild($no->createCDATASection($valueAsCData));
    }

    return $child;
  }

  /**
   * Adds a time value as a child and properly formats the time.
   *
   * @param string $name
   * @param mixed $value This is sent to DateObject class.
   * @param string $namespace 
   *
   * @see  DateObject
   *
   * This function depends on the date module being enabled.
   *
   * @see  https://www.drupal.org/project/date
   */
  public function addDateChild($name, $value, $namespace = NULL) {
    $value = $value instanceof DateObject ? $value : new DateObject($value);

    return $this->addChild($name, $value->format('r'), $namespace);
  }

  /**
   * Extends to allow chaining of add attribute
   *
   * e.g. $xml->addAttribute('size', 'large')->addAttribute('color', 'blue')
   */
  public function addAttribute($name, $value = NULL, $namespace = NULL) {
    parent::addAttribute($name, $value, $namespace);
    // switch (func_num_args()) {
    //   case 1:
    //     parent::addAttribute($name);
    //     break;
    //   case 2:
    //     parent::addAttribute($name, $value);
    //     break;
    //   case 3:
    //     parent::addAttribute($name, $value, $namespace);
    //     break;
    // }

    return $this;
  }

  // /**
  //  * Renders to xml, counterpart to the CDATA issue
  //  */
  // public function asXML($filename = NULL) {
  //   switch (func_num_args()) {
  //     case 0:
  //       $string = parent::asXML();
  //       break;

  //     case 1:
  //       $string = parent::asXML($filename);
  //       break;
  //   }

  //   return $string;
  // }

  protected static function setConfig($name, $value) {
    self::$config[$name] = $value;
  }

  protected static function getConfig($name, $default = NULL) {
    $value = self::$config[$name];

    return isset($value) ? $value : $default;
  }

  /**
   * Modifies a string by wrapping the CDATA when needed.
   *
   * @param  string &$value
   */
  protected static function wrapCDataIfNeeded(&$value, $force = FALSE) {
    if (strpos($value, '<![CDATA[') === FALSE
      && ($force || preg_match("/[<&>\"']/", $value))) {
      $value = "<![CDATA[{$value}]]>";
    }
  }

}

